# -*- coding: utf-8 -*-

# Kyushu University
# Developed by Hitoshi Teshima and Akihiko Sayo
# Ver 0.1.0
# 2019/09/23 All Right Reserved Kawasaki Lab.

import sys
import os
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
import glob
from PIL import Image
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from ProjectFile import ProjectFile as pf


class GUI(QMainWindow):

    def __init__(self):
        super().__init__()
        self.config = {}
        self.config['projectRoot'] = 'C:/code/Calibration/ARCalib/ARCalib_GUI/20190717_ARCalib/'
        self.config['captureList'] = []
        self.config['Projection_img_name'] = 'ARMarkerPattern22.png'
        self.config['cam'] = {}
        self.config['proj'] = {}
        self.config['cam']['AR_thresh'] = 80
        self.config['cam']['thresh_first'] = 20
        self.config['cam']['thresh_interval'] = 5
        self.config['cam']['thresh_repeat_num'] = 10
        self.config['cam']['calib_mode'] = 0
        self.config['cam']['use_center'] = 1
        self.config['cam']['Adaptive_thresh'] = 1
        self.config['cam']['boad_rotation'] = 0
        self.config['cam']['Minimum_number'] = 4
        self.config['cam']['MARK_SIZE'] = 0.0395
        self.config['cam']['MARK_SPACE_X'] = 0.0055
        self.config['cam']['MARK_SPACE_Y'] = 0.0035
        self.config['cam']['distflag'] = 1
        self.config['cam']['width'] = 1600
        self.config['cam']['height'] = 1200
        self.config['proj']['AR_thresh'] = 90
        self.config['proj']['thresh_first'] = 50
        self.config['proj']['thresh_interval'] = 3
        self.config['proj']['thresh_repeat_num'] = 30
        self.config['proj']['calib_mode'] = 1
        self.config['proj']['use_center'] = 1
        self.config['proj']['Adaptive_thresh'] = 1
        self.config['proj']['boad_rotation'] = 0
        self.config['proj']['Minimum_number'] = 4
        self.config['proj']['MARK_SIZE'] = 0.0395
        self.config['proj']['MARK_SPACE_X'] = 0.0055
        self.config['proj']['MARK_SPACE_Y'] = 0.0035
        self.config['proj']['distflag'] = 0
        self.config['proj']['width'] = 1024
        self.config['proj']['height'] = 768
        self.initUI()
        self.projectName = None

    def initUI(self):
        self.statusBar()
        self.createActions()
        self.createMenus()

        # フィギュアウィジェット(右側の画像描画)
        self.FigureWidget1 = QWidget(self)
        # FigureWidgetにLayoutを追加
        self.FigureLayout1 = QHBoxLayout(self.FigureWidget1)
        # Marginを消す
        self.FigureLayout1.setContentsMargins(0, 0, 0, 0)
        # ファイルリスト(左側の各画像の名前、チェックボックス)
        self.FileList = QListWidget(self)

        # Figureを作成
        self.Figure1 = plt.figure()
        self.Figure1.tight_layout()
        # FigureをFigureCanvasに追加
        self.FigureCanvas1 = FigureCanvas(self.Figure1)
        # LayoutにFigureCanvasを追加
        self.FigureLayout1.addWidget(self.FigureCanvas1)

        # 各処理用のボタン配置
        pos = 10
        btn_new_proj = QPushButton('New project', self)
        btn_new_proj.move(pos, 40)
        btn_new_proj.clicked.connect(self.newProject)

        pos += 100
        btn_open_proj = QPushButton('Open project', self)
        btn_open_proj.move(pos, 40)
        btn_open_proj.clicked.connect(self.openProject)

        pos += 100
        btn_open_img = QPushButton('Open images', self)
        btn_open_img.move(pos, 40)
        btn_open_img.clicked.connect(self.openImages)

        pos += 100
        btn_iso = QPushButton('isolation', self)
        btn_iso.move(pos, 40)
        btn_iso.clicked.connect(self.isoImg)

        pos += 100
        btn_dct_cam = QPushButton('detect Cam', self)
        btn_dct_cam.move(pos, 40)
        btn_dct_cam.clicked.connect(self.detectCam)

        pos += 100
        btn_dct_proj = QPushButton('detect Proj', self)
        btn_dct_proj.move(pos, 40)
        btn_dct_proj.clicked.connect(self.detectProj)

        pos += 100
        btn_calib_cam = QPushButton('Cam self calib', self)
        btn_calib_cam.move(pos, 40)
        btn_calib_cam.clicked.connect(self.selfCalibCam)

        pos += 100
        btn_calib_cam = QPushButton('Proj self calib', self)
        btn_calib_cam.move(pos, 40)
        btn_calib_cam.clicked.connect(self.selfCalibProj)

        pos += 100
        btn_calib_cam = QPushButton('Undistort point', self)
        btn_calib_cam.move(pos, 40)
        btn_calib_cam.clicked.connect(self.undistortCam)

        pos += 100
        btn_calib_stereo = QPushButton('Stereo calib', self)
        btn_calib_stereo.move(pos, 40)
        btn_calib_stereo.clicked.connect(self.stereoCalib)

        pos += 100
        btn_calib_stereo = QPushButton('Convert openCV to I3D', self)
        btn_calib_stereo.move(pos, 40)
        btn_calib_stereo.clicked.connect(self.convertOpencv2i3d)

        # 配置
        self.setGeometry(50, 50, 1500, 1220)
        self.FigureWidget1.setGeometry(200, 80, 1200, 1200)
        self.FileList.setGeometry(0, 80, 200, 1200)

        self.setWindowTitle('AR Calibration')
        self.show()

    def resizeEvent(self, event):
        QMainWindow.resizeEvent(self, event)

    # -------------メニューバーの関数------------- #
    def newProject(self):
        pf.createNewProject(self, config=self.config)

    def openProject(self):
        pf.openJson(self, config=self.config)

    def saveProject(self):
        pf.saveJson(self, config=self.config)

    def openImages(self):
        # エクスプローラーを表示、フォルダ選択
        self.dirName = QFileDialog.getExistingDirectory(self, 'Open Directory', os.getcwd()) + '/'
        if self.dirName == '': return
        self.loadImages(self.FigureCanvas1, state='capture')
        
        # captureフォルダをプロジェクトファイルの場所にコピー
        import shutil
        if os.path.exists(self.config['projectRoot']+'capture/') is False:
            shutil.copytree(self.cap_dir, self.config['projectRoot']+'capture/')

    def loadImages(self, FigureCanvas, state):
        # 2回目以降の読み込みのため中身をクリア
        self.FileList.clear()
        # ファイルを配置
        self.setFileList(state=state)
        # 画像
        if self.FileList.item(0) is not None:
            self.fileName = self.FileList.item(0).text()
        else:
            QMessageBox.information(self, 'AR Caribration', 'There is no image in ' + self.dirName)
            return
        # 画像を読み込む
        self.fileName = self.FileList.item(0).text()
        self.loadImageFile()
        # Figureの初期化
        self.initFigure()
        FigureCanvas.draw()
        # ファイルを変更した時のイベント
        self.FileList.itemSelectionChanged.connect(lambda: self.fileListChanged(FigureCanvas))

    # Figureの初期化
    def initFigure(self):
        self.axis = self.Figure1.add_subplot(1, 1, 1)
        self.axis_image = self.axis.imshow(self.image, cmap='gray')
        plt.axis('off')

    def setFileList(self, state):
        # ファイルの読み込み
        self.cap_dir = './' + os.path.relpath(self.dirName) + '/'
        pngFiles = glob.glob(self.dirName + '/*.' + 'png')
        jpgFiles = glob.glob(self.dirName + '/*.' + 'jpg')
        bmpFiles = glob.glob(self.dirName + '/*.' + 'bmp')
        self.Files = pngFiles + jpgFiles + bmpFiles
        sorted(self.Files)
        # ファイルリストに追加
        for i in range(len(self.Files)):
            item = QListWidgetItem(os.path.basename(self.Files[i]))
            # could be Qt.Unchecked; setting it makes the check appear
            item.setCheckState(Qt.Checked)
            self.FileList.addItem(item)
            if state is 'capture': self.config['captureList'].append({'name':os.path.basename(self.Files[i]), 'id':i, 'TF':True})

    # 選択画像を変更した時の関数
    def fileListChanged(self, FigureCanvas):
        if not self.FileList.selectedItems(): return
        # 選択している画像の名前を取得
        self.fileName = self.FileList.selectedItems()[0].text()
        # 画像を読み込み
        self.loadImageFile()
        # Figureを更新
        self.updateFigure(FigureCanvas)

    # 画像ファイルを読み込む
    def loadImageFile(self):
        # 画像を開く
        image = Image.open(self.dirName + self.fileName)
        # numpy.ndarrayに
        self.image = np.asarray(image)

    # Figureを更新
    def updateFigure(self, FigureCanvas):
        self.axis_image.set_data(self.image)
        FigureCanvas.draw()

    def isoImg(self):
        from util import iso_img
        # in_files = glob.glob(self.cap_dir+'/*.*')
        os.makedirs(self.config['projectRoot']+'iso/', exist_ok=True)
        # iso_img.iso_img(files=in_files, indir='', outdir=self.iso_dir)
        iso_img.isoImg(self)
        
        # 分離した画像を表示
        self.dirName = self.config['projectRoot']+'iso/'
        self.loadImages(self.FigureCanvas1, state='iso')

    def detectCam(self, mode='cam'):
        from util import SubWindow
        subwindow = SubWindow.DetectARConfigWindow(mode='cam', config=self.config)
        subwindow.show()
        if(subwindow.isStarted is False) : return

        from util import detectAR
        os.makedirs(self.config['projectRoot']+'test/', exist_ok=True)
        os.makedirs(self.config['projectRoot']+'info/', exist_ok=True)
        os.makedirs(self.config['projectRoot']+'cam/point/', exist_ok=True)
        os.makedirs(self.config['projectRoot']+'cam/detect_AR_image/', exist_ok=True)

        result = detectAR.detectAR(self, mode='cam')
        # 検出した画像を表示
        self.dirName = self.config['projectRoot']+'cam/detect_AR_image/'
        self.loadImages(self.FigureCanvas1, state='detectARCam')

        self.checkFileState(mode='cam')

    def detectProj(self):
        from util import SubWindow
        subwindow = SubWindow.DetectARConfigWindow(mode='proj', config=self.config)
        subwindow.show()
        if(subwindow.isStarted is False) : return

        from util import detectAR
        os.makedirs(self.config['projectRoot']+'proj/point/', exist_ok=True)
        os.makedirs(self.config['projectRoot']+'proj/detect_AR_image/', exist_ok=True)

        result = detectAR.detectAR(self, mode='proj')
        # 検出した画像を表示
        self.dirName = self.config['projectRoot']+'proj/detect_AR_image/'
        self.loadImages(self.FigureCanvas1, state='detectARProj')

        self.checkFileState(mode='proj')

    def selfCalibCam(self):
        from util import SubWindow
        subwindow = SubWindow.DistFlagWindow(config=self.config, FileList=self.FileList)
        subwindow.show()
        if(subwindow.isStarted is False) : return

        from util import self_calib
        os.makedirs(self.config['projectRoot']+'cam/param/', exist_ok=True)
        os.makedirs(self.config['projectRoot']+'cam/reproj/', exist_ok=True)

        np.savetxt(self.config['projectRoot']+'validID.txt', self.validID.reshape(-1, 1), fmt='%d')
        self_calib.selfCalib(self, mode='cam')

        # 再投影した画像を表示
        self.dirName = self.config['projectRoot']+'cam/reproj/'
        self.loadImages(self.FigureCanvas1, state='selfCalibCam')

    def selfCalibProj(self):
        from util import self_calib
        os.makedirs(self.config['projectRoot']+'proj/param/', exist_ok=True)
        os.makedirs(self.config['projectRoot']+'proj/reproj/', exist_ok=True)

        np.savetxt(self.config['projectRoot']+'validID.txt', self.validID.reshape(-1, 1), fmt='%d')
        self_calib.selfCalib(self, mode='proj')

        # 再投影した画像を表示
        self.dirName = self.config['projectRoot']+'proj/reproj/'
        self.loadImages(self.FigureCanvas1, state='selfCalibProj')

    def stereoCalib(self):
        from util import stereo_calib
        os.makedirs(self.config['projectRoot']+'param/', exist_ok=True)
        stereo_calib.stereoCalib(self)

    def convertOpencv2i3d(self):
        from util import intrinsic_cv2i3d
        from util import extrinsic_cv2i3d
        intrinsic_cv2i3d.intrinsicCV2I3D(self, mode='cam')
        intrinsic_cv2i3d.intrinsicCV2I3D(self, mode='proj')
        extrinsic_cv2i3d.extrinsicCV2I3D(self)

    def undistortCam(self):
        from util import undist_point
        os.makedirs(self.config['projectRoot']+'cam/point_ud/', exist_ok=True)
        result = undist_point.undistPoint(self, mode='cam')
        if result == 0:
            print('undist DONE!')

    def checkFileState(self, mode):
        if mode == 'cam':
            state = np.loadtxt(self.config['projectRoot']+'cam/point/cam_true_num.txt')
        elif mode == 'proj':
            state = np.loadtxt(self.config['projectRoot']+'proj/point/proj_true_num.txt')
        else:
            state = np.loadtxt(self.config['projectRoot']+'proj/point/proj_true_num.txt')
        self.validID = np.array([i for i in range(len(self.config['captureList'])) if self.config['captureList'][i]['TF'] is True])
        self.validID = np.intersect1d(state, self.validID)

    # アクションの割り当て
    def createActions(self):
        self.newProjAct = QAction('&New Project', self, shortcut='', triggered=self.newProject)
        self.openProjAct = QAction('&Open Project', self, shortcut='', triggered=self.openProject)
        self.saveProjAct = QAction('&Save Project', self, shortcut='Ctrl+S', triggered=self.saveProject)
        self.openImageAct = QAction('&Open Images', self, shortcut='', triggered=self.openImages)
        self.isoimgAct = QAction('Isolation Images', self, shortcut='', triggered=self.isoImg)
        self.exitAct = QAction('&Exit', self, shortcut='', triggered=self.close)

    # メニューバーの配置
    def createMenus(self):
        self.fileMenu = QMenu('&File', self)
        self.fileMenu.addAction(self.newProjAct)
        self.fileMenu.addAction(self.openProjAct)
        self.fileMenu.addAction(self.saveProjAct)
        self.fileMenu.addSeparator()
        self.fileMenu.addAction(self.exitAct)

        self.toolMenu = QMenu('&Tools', self)
        self.toolMenu.addAction(self.openImageAct)
        self.toolMenu.addAction(self.isoimgAct)

        self.menuBar().addMenu(self.fileMenu)
        self.menuBar().addMenu(self.toolMenu)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    ex = GUI()
    sys.exit(app.exec_())
