# -*- coding: utf-8 -*-

# Kyushu University
# Developed by Hitoshi Teshima and Akihiko Sayo
# Ver 0.1.0
# 2019/09/23 All Right Reserved Kawasaki Lab.

import sys, os
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *

class DetectARConfigWindow:
    AR_thresh_edit         = None
    thresh_first_edit      = None
    thresh_interval_edit   = None
    thresh_repeat_num_edit = None
    calib_mode_edit        = None
    use_center_edit        = None
    Adaptive_thresh_edit   = None
    boad_rotation_edit     = None
    Minimum_number_edit    = None
    MARK_SIZE_edit         = None
    MARK_SPACE_X_edit      = None
    MARK_SPACE_Y_edit      = None

    def __init__(self, mode, config):
        # パラメータのラベルウィジェット
        AR_thresh           = QLabel('AR_thresh')
        thresh_first        = QLabel('thresh_first')
        thresh_interval     = QLabel('thresh_interval')
        thresh_repeat_num   = QLabel('thresh_repeat_num')
        calib_mode          = QLabel('calib_mode')
        use_center          = QLabel('use_center')
        Adaptive_thresh     = QLabel('Adaptive_thresh')
        boad_rotation       = QLabel('boad_rotation')
        Minimum_number      = QLabel('Minimum_number')
        MARK_SIZE           = QLabel('MARK_SIZE')
        MARK_SPACE_X        = QLabel('MARK_SPACE_X')
        MARK_SPACE_Y        = QLabel('MARK_SPACE_Y')

        # パラメータのラインエディットウィジェット
        self.AR_thresh_edit          = QLineEdit()
        self.thresh_first_edit       = QLineEdit()
        self.thresh_interval_edit    = QLineEdit()
        self.thresh_repeat_num_edit  = QLineEdit()
        self.calib_mode_edit         = QLineEdit()
        self.use_center_edit         = QLineEdit()
        self.Adaptive_thresh_edit    = QLineEdit()
        self.boad_rotation_edit      = QLineEdit()
        self.Minimum_number_edit     = QLineEdit()
        self.MARK_SIZE_edit          = QLineEdit()
        self.MARK_SPACE_X_edit       = QLineEdit()
        self.MARK_SPACE_Y_edit       = QLineEdit()

        # デフォルトの値を設定
        self.AR_thresh_edit          .setText(str(config[mode]['AR_thresh']))
        self.thresh_first_edit       .setText(str(config[mode]['thresh_first']))
        self.thresh_interval_edit    .setText(str(config[mode]['thresh_interval']))   
        self.thresh_repeat_num_edit  .setText(str(config[mode]['thresh_repeat_num']))  
        self.calib_mode_edit         .setText(str(config[mode]['calib_mode']))
        self.use_center_edit         .setText(str(config[mode]['use_center']))
        self.Adaptive_thresh_edit    .setText(str(config[mode]['Adaptive_thresh']))
        self.boad_rotation_edit      .setText(str(config[mode]['boad_rotation']))
        self.Minimum_number_edit     .setText(str(config[mode]['Minimum_number']))
        self.MARK_SIZE_edit          .setText(str(config[mode]['MARK_SIZE']))
        self.MARK_SPACE_X_edit       .setText(str(config[mode]['MARK_SPACE_X']))
        self.MARK_SPACE_Y_edit       .setText(str(config[mode]['MARK_SPACE_Y']))
        
        # 格子状の配置を作り、各ウィジェットのスペースを空ける
        grid = QGridLayout()
        grid.setSpacing(10)

        # ラベルの位置設定
        grid.addWidget(AR_thresh         , 1, 0)
        grid.addWidget(thresh_first      , 2, 0)
        grid.addWidget(thresh_interval   , 3, 0)
        grid.addWidget(thresh_repeat_num , 4, 0)
        grid.addWidget(calib_mode        , 5, 0)
        grid.addWidget(use_center        , 6, 0)
        grid.addWidget(Adaptive_thresh   , 7, 0)
        grid.addWidget(boad_rotation     , 8, 0)
        grid.addWidget(Minimum_number    , 9, 0)
        grid.addWidget(MARK_SIZE         , 10, 0)
        grid.addWidget(MARK_SPACE_X      , 11, 0)
        grid.addWidget(MARK_SPACE_Y      , 12, 0)
        
        # 入力欄の位置設定
        grid.addWidget(self.AR_thresh_edit         , 1, 1)
        grid.addWidget(self.thresh_first_edit      , 2, 1)
        grid.addWidget(self.thresh_interval_edit   , 3, 1)
        grid.addWidget(self.thresh_repeat_num_edit , 4, 1)
        grid.addWidget(self.calib_mode_edit        , 5, 1)
        grid.addWidget(self.use_center_edit        , 6, 1)
        grid.addWidget(self.Adaptive_thresh_edit   , 7, 1)
        grid.addWidget(self.boad_rotation_edit     , 8, 1)
        grid.addWidget(self.Minimum_number_edit    , 9, 1)
        grid.addWidget(self.MARK_SIZE_edit         , 10, 1)
        grid.addWidget(self.MARK_SPACE_X_edit      , 11, 1)
        grid.addWidget(self.MARK_SPACE_Y_edit      , 12, 1)

        # 開始ボタン
        start_button = QPushButton('Start to detect')
        start_button.clicked.connect(lambda: self.updateParam(mode=mode, config=config))

        # レイアウト
        self.w = QDialog()
        vbox = QVBoxLayout()
        vbox.addLayout(grid)
        vbox.addWidget(start_button)
        self.w.setLayout(vbox) 
        self.w.setGeometry(600, 200, 200, 300)
        self.w.setWindowTitle('detect AR Cam Configuration')    

        # スタートボタンが押されたかどうか
        self.isStarted = False

    def updateParam(self, mode, config):
        config[mode]['AR_thresh']          = int(self.AR_thresh_edit.text())
        config[mode]['thresh_first']       = int(self.thresh_first_edit.text())
        config[mode]['thresh_interval']    = int(self.thresh_interval_edit.text())
        config[mode]['thresh_repeat_num']  = int(self.thresh_repeat_num_edit.text())
        config[mode]['calib_mode']         = int(self.calib_mode_edit.text())
        config[mode]['use_center']         = int(self.use_center_edit.text())
        config[mode]['Adaptive_thresh']    = int(self.Adaptive_thresh_edit.text())
        config[mode]['boad_rotation']      = int(self.boad_rotation_edit.text())
        config[mode]['Minimum_number']     = int(self.Minimum_number_edit.text())
        config[mode]['MARK_SIZE']          = float(self.MARK_SIZE_edit.text())
        config[mode]['MARK_SPACE_X']       = float(self.MARK_SPACE_X_edit.text())
        config[mode]['MARK_SPACE_Y']       = float(self.MARK_SPACE_Y_edit.text())
        self.isStarted = True
        self.w.close()

    def show(self):
        self.w.exec_()

class DistFlagWindow:
    def __init__(self, config, FileList):
        dist_flag = QLabel('Distortion Calib')
        
        radio1 = QRadioButton('Distortion Calib ON')
        radio2 = QRadioButton('Distortion Calib OFF')
        self.group = QButtonGroup()
        self.group.addButton(radio1, 1)
        self.group.addButton(radio2, 2)
        radio1.toggle()

        start_button = QPushButton('Start selfcalib')
        start_button.clicked.connect(lambda: self.updateParam(config=config, FileList=FileList))
        grid = QGridLayout()
        grid.setSpacing(10)
        grid.addWidget(radio1 , 1, 0)
        grid.addWidget(radio2, 2, 0)
        
        self.w = QDialog()
        vbox = QVBoxLayout()
        vbox.addLayout(grid)
        vbox.addWidget(start_button)
        self.w.setLayout(vbox) 
        self.w.setGeometry(600, 300, 100, 100)
        self.w.setWindowTitle('distortion flag')    
        # スタートボタンが押されたかどうか
        self.isStarted = False

    def updateParam(self, config, FileList):
        if self.group.checkedId() is 1: config['cam']['distflag'] = 1
        else: config['cam']['distflag'] = 0
        for i in range(len(config['captureList'])):
            if FileList.item(i).checkState() == Qt.Checked: config['captureList'][i]['TF'] = True
            else: config['captureList'][i]['TF'] = False
        self.isStarted = True
        self.w.close()

    def show(self):
        self.w.exec_()